/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2014 - 2017 Dries K. Aka Dries007
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package net.dries007.holoInventory.client.renderers;

import net.dries007.holoInventory.client.ClientEventHandler;
import net.minecraft.client.Minecraft;
import net.minecraft.client.multiplayer.WorldClient;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.RenderItem;
import net.minecraft.client.renderer.entity.RenderManager;
import net.minecraft.client.renderer.tileentity.TileEntityRendererDispatcher;
import net.minecraft.client.resources.I18n;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.ArrayList;
import java.util.List;

@SideOnly(Side.CLIENT)
public class InventoryRenderer implements IRenderer
{
    private final String name;
    private final List<ItemStack> stacks;

    public InventoryRenderer(final String name, List<ItemStack> input)
    {
        // Minecraft & Localization. One giant clusterfuck.
        String tmp = I18n.func_135052_a(name);
        if (!tmp.equals(name)) this.name = tmp;
        else
        {
            String name2 = name + ".name";
            tmp = I18n.func_135052_a(name2);
            if (!tmp.equals(name2)) this.name = tmp;
            else this.name = name;
        }

        this.stacks = new ArrayList<>(input.size());
        for (ItemStack stack : input)
        outer: {
            if (stack == null) continue;
            for (ItemStack stack2 : stacks)
            {
                if (!ItemStack.func_77970_a(stack, stack2) || !ItemStack.func_179545_c(stack, stack2)) continue;
                stack2.func_190917_f(stack.func_190916_E());
                break outer;
            }
            this.stacks.add(stack);
        }
    }

    @Override
    public void render(WorldClient world, RayTraceResult hit, Vec3d pos)
    {
        Minecraft mc = Minecraft.func_71410_x();
        RenderManager rm = mc.func_175598_ae();
        RenderItem ri = mc.func_175599_af();

        GlStateManager.func_179137_b(pos.field_72450_a - TileEntityRendererDispatcher.field_147554_b, pos.field_72448_b - TileEntityRendererDispatcher.field_147555_c, pos.field_72449_c - TileEntityRendererDispatcher.field_147552_d);

        GlStateManager.func_179114_b(-rm.field_78735_i, 0.0F, 0.5F, 0.0F);
        GlStateManager.func_179114_b(rm.field_78732_j, 0.5F, 0.0F, 0.0F);
        GlStateManager.func_179137_b(0, 0, -0.5);

        double d = pos.func_72438_d(new Vec3d(TileEntityRendererDispatcher.field_147554_b, TileEntityRendererDispatcher.field_147555_c, TileEntityRendererDispatcher.field_147552_d));

        if (d < 1.75) return;
        GlStateManager.func_179139_a(d * 0.2, d * 0.2, d * 0.2);

        int cols;
        if (stacks.size() <= 9) cols = stacks.size();
        else if (stacks.size() <= 27) cols = 9;
        else if (stacks.size() <= 54) cols = 11;
        else if (stacks.size() <= 90) cols = 14;
        else if (stacks.size() <= 109) cols = 18;
        else cols = 21;
        int rows = 1 + ((stacks.size() % cols == 0) ? (stacks.size() / cols) - 1 : stacks.size() / cols);

        if (rows > 4) GlStateManager.func_179139_a(0.8, 0.8, 0.8);

        // Draw name, with depth disabled
        {
            GlStateManager.func_179094_E();
            GlStateManager.func_179123_a();
            GlStateManager.func_179114_b(180, 0, 0, 1);
            GlStateManager.func_179137_b(0, -0.6f -0.4f * (rows/2.0), 0);
            //GlStateManager.translate(0, -1 - 0.5f * (rows/2.0), 0);
            GlStateManager.func_179139_a(0.03, 0.03, 0.03);
            int w = mc.field_71466_p.func_78256_a(name);
            GlStateManager.func_179097_i();
            mc.field_71466_p.func_78276_b(name,  -w/2, 0, 0xFFFFFF);
            GlStateManager.func_179126_j();
            GlStateManager.func_179099_b();
            GlStateManager.func_179121_F();
        }

        int r = 0;
        int c = 0;
        for (final ItemStack stack : stacks)
        {
            RenderHelper.renderStack(ri, stack, cols, c, rows, r);
            if (++c == cols)
            {
                r++;
                c = 0;
            }
        }
        // Draw stack sizes later, to draw over the items (disableDepth)
        r = 0;
        c = 0;
        GlStateManager.func_179097_i();
        for (final ItemStack stack : stacks)
        {
            RenderHelper.renderName(mc.field_71466_p, stack, cols, c, rows, r, ClientEventHandler.TEXT_COLOR);
            if (++c == cols)
            {
                r++;
                c = 0;
            }
        }
        GlStateManager.func_179126_j();
    }

    @Override
    public boolean shouldRender()
    {
        return stacks.size() != 0;
    }
}
